//+------------------------------------------------------------------+
//|                                                        Trade.mqh |
//|                        Copyright 2015, Pham Thanh Tu 0917388656. |
//|                                 http://laptrinhrobotforex.edu.vn |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015.01..02, Pham Thanh Tu 0917388656."
#property link      "http://goldandforex168.com"
#property strict

//--------------------------------------------------------------------
// Trade.mqh
//
//--------------------------------------------------------------------
// Trade function. 
   // Trad_Oper - trade operation type:
   // 10 - opening  Buy 
   // 11 - closing  Buy
   // 20 - opening  Sell   
   // 21 - closing  Sell
   // 0  - no important criteria available.
   // 1  - doing candle trailing if enable
   // -1 - another symbol is used  
//--------------------------------------------------------------- 1 --
int Trade(int Trad_Oper)              // User-defined function
{
     int Ticket = -1; // Ticket of new order     
     switch(Trad_Oper)
     {
          //------------------------------------- 2 --
          case 10:
               if(TradeTotalMarket() == 0)
               {
                    if(AllowOpenPosition(OP_BUY) && TradeOnce(10))
                    {
                         double NewOrderLot     = NewOrderLot();
                         int NewOrderStopLoss   = NewOrderStopLoss();
                         int NewOrderTakeProfit = NewOrderTakeProfit();
                         Ticket  = Open_Ord(OP_BUY,NewOrderLot,NewOrderStopLoss,NewOrderTakeProfit);  //Open first Buy                          
                         if(Ticket > 0) TradeOnce(10,true); //Update ActonOnce
                    }
               }
               return(Ticket); // Having traded, leave
          //------------------------------------ 3 --
          case 20:
               if(TradeTotalMarket() == 0)
               {
                    if(AllowOpenPosition(OP_SELL) && TradeOnce(20))
                    {
                         double NewOrderLot     = NewOrderLot();
                         int NewOrderStopLoss   = NewOrderStopLoss();
                         int NewOrderTakeProfit = NewOrderTakeProfit();
                         Ticket = Open_Ord(OP_SELL,NewOrderLot,NewOrderStopLoss,NewOrderTakeProfit); //Open first Sell
                         if(Ticket > 0 ) TradeOnce(20,true); //Update ActonOnce
                    }
               }
               return(Ticket);          //Having traded, leave
     }  
     return(0);
}


bool AllowOpenPosition(int Tip)
{    
     if(!AllowTradesByTime())
     {
          return(false);
     } 
     if(MinLot > LotSize)
     {    
          Print("Your account requires Min Lot " + (string)MinLot);
          Print("The Robot requires Min Lot "    + (string)LotSize);
          Print("Trading is rejected");
          Print("Contact your broker to setup the account");
          return(false);
     }     
     //-----------------------------------------
     return(true);     
}


bool AllowTradesByTime()
{
     int Current_Time = TimeHour(TimeCurrent());
     if (StartHour == 0) StartHour = 24; if (EndHour == 0) EndHour = 24; if (Current_Time == 0) Current_Time = 24;
 
     if ( StartHour < EndHour )
     {
          if ( (Current_Time < StartHour) || (Current_Time >= EndHour) ) return(false);
     }     
 
     if ( StartHour > EndHour )
     {
          if ( (Current_Time < StartHour) && (Current_Time >= EndHour) ) return(false);
     }
     
     //Minute
     int Current_Minute = TimeMinute(TimeCurrent());
     if (StartMinute == 0) StartMinute = 60; if (EndMinute == 0) EndMinute = 60; if (Current_Minute == 0) Current_Time = 60;
     if ( StartMinute < EndMinute )
     {
          if ( (Current_Minute < StartMinute) || (Current_Minute >= EndMinute) ) return(false);
     }     
 
     if ( StartMinute > EndMinute )
     {
          if ( (Current_Minute < StartMinute) && (Current_Minute >= EndMinute) ) return(false);
     }          
     
     return(true);
}


bool IsValidPrice(int Tip, double Price)
{
     if(TradeTotalMarket() == 0) return(true);
     
     int Type = IsLongPosition(Tip) ? OP_BUY : OP_SELL;
     
     double HighPrice = 0; int OrdType = -1;     
     for(int i = 1; i <= Mas_Ord_New[0][0]; i++)
     {
          OrdType = (int)Mas_Ord_New[i][6];
          if(IsMarketOrder(OrdType) && OrdType == Type )
          {
               if(HighPrice < Mas_Ord_New[i][1]) HighPrice = Mas_Ord_New[i][1];
          }     
     }  
     double LowPrice = HighPrice;
     for(int i = 1; i <= Mas_Ord_New[0][0]; i++)
     {
          OrdType = (int)Mas_Ord_New[i][6];
          if(IsMarketOrder(OrdType) && OrdType == Type )
          {
               if(LowPrice > Mas_Ord_New[i][1]) LowPrice = Mas_Ord_New[i][1];
          }     
     }
     if(HighPrice >= Price && Price >= LowPrice) 
     {
          return(false);
     }     
     
     return(true);      
}
